/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * JFlex 1.3.5                                                             *
 * Copyright (C) 1998-2001  Gerwin Klein <lsf@jflex.de>                    *
 * All rights reserved.                                                    *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License. See the file      *
 * COPYRIGHT for more information.                                         *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                 *
 *                                                                         *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

package JFlex;

/**
 * Stores an intervall of characters together with the character class
 *
 * A character belongs to an intervall, if its Unicode value is greater than or equal
 * to the Unicode value of <CODE>start</code> and smaller than or euqal to the Unicode
 * value of <CODE>end</code>.
 *
 * All characters of the intervall must belong to the same character class.
 *
 * @author Gerwin Klein
 * @version JFlex 1.3.5, $Revision: 1.14 $, $Date: 2001/10/08 10:07:57 $
 */
public class CharClassIntervall {

  /**
   * The first character of the intervall
   */
  int start;

  /**
   * The last character of the intervall
   */
  int end;

  /**
   * The code of the class all characters of this intervall belong to.
   */
  int charClass;
  

  /**
   * Creates a new CharClassIntervall from <CODE>start</code> to <CODE>end</code>
   * that belongs to character class <CODE>charClass</code>.
   *
   * @param start         The first character of the intervall
   * @param end           The last character of the intervall  
   * @param charClass     The code of the class all characters of this intervall belong to.
   */
  public CharClassIntervall(int start, int end, int charClass) {
    this.start = start;
    this.end = end;
    this.charClass = charClass;
  }

  /**
   * returns string representation of this class-intervall
   */
  public String toString() {
    return "["+start+"-"+end+"="+charClass+"]";
  }
}
